# R CODE FOR REGRESSION ANALYSIS OF INTENSIFIERS DATA FROM THE OLD BAILEY CORPUS 2.0
#--------------------------------------------------------------
# The code was created by Lukas Sönning, University of Bamberg,
# and is produced here in the version it was run by
# Ewa Jonsson, Mid Sweden University, in November 2019 through November 2020.

# The code file is supplementary material to the following publication:
# Claudia Claridge, Ewa Jonsson, and Merja Kytö. 2024. 
# Intensifiers in Late Modern English: A sociopragmatic approach to courtroom discourse. 
# Cambridge: Cambridge University Press.
#--------------------------------------------------------------

# missing data with brms

library(brms) #run these three lines to load packages for every session
library(lattice)
library(effects)

#-----------------------------
# before the analysis, convert the relevant .xlsx data file to .csv
# the code below pertains to .csv files
maximizers <- read.csv2(file.choose()) #use this for the maximizers file; similarly for the data files of the other intensifier categories
View(maximizers)
str(maximizers)


# INSPECT
#--------------------------------------------------------------

# screen data
xtabs(~maximizers$gender)
xtabs(~maximizers$role)
xtabs(~maximizers$class)

# missing data
round(sum(maximizers$gender=="") / nrow(maximizers), 3)*100
round(sum(maximizers$role=="") / nrow(maximizers), 3)*100
round(sum(maximizers$class=="") / nrow(maximizers), 3)*100


# RECODE
#--------------------------------------------------------------

# recode gender -> female
maximizers$female = ifelse(maximizers$gender == "f", 1, 0)
maximizers$female = ifelse(maximizers$gender == "", NA, maximizers$female)

# recode class -> higher
maximizers$higher = ifelse(maximizers$class == "higher (1-5)", 1, 0)
maximizers$higher = ifelse(maximizers$class == "", NA, maximizers$higher)

# add variable with logged counts of total words spoken
maximizers$log_wordcount = log(maximizers$wordCount)

# rescale year: center 1800 -> 0; sd = 50
maximizers$year_c = (maximizers$year - 1800)/50
maximizers$rate = maximizers$Maximizers/maximizers$wordCount


# GRAPHICAL INSPECTION
#--------------------------------------------------------------

histogram(maximizers$year, xlab="maximizers") #change main and xlab depending upon intensifier category
xyplot(rate ~ year, data=maximizers,alpha=.2, col=1)
xyplot(rate*100000 ~ year, data=maximizers,alpha=.2, col=1)
bwplot(sqrt(rate) ~ gender, data=maximizers)
bwplot(sqrt(rate) ~ class, data=maximizers)
bwplot(sqrt(rate) ~ role, data=maximizers)


# RECODE TO REPRODUCE ANALYSIS BY INGER
#--------------------------------------------------------------

# recode year -> 5 periods
maximizers$year_cat = factor(ifelse(maximizers$year <= 1759, "1720-1759",
									ifelse(maximizers$year >= 1760 & maximizers$year <= 1799, "1760-1799",
										   ifelse(maximizers$year >= 1800 & maximizers$year <= 1839, "1800-1839",
										   	   ifelse(maximizers$year >= 1840 & maximizers$year <= 1879, "1840-1879",
										   	   	   ifelse(maximizers$year >= 1880 & maximizers$year <= 1913, "1880-1913", NA))))))

# recode role
maximizers$role2 = maximizers$role
maximizers$role2[maximizers$role == ""] <- NA
maximizers$role2[maximizers$role == "Interpreter"] <- NA
maximizers$role2 = factor(maximizers$role2, levels = c("Defendant", "Witness", "Judge","Victim", "Lawyer"))
xtabs(~maximizers$role2)

# recode class
maximizers$class2 = maximizers$class
maximizers$class2[maximizers$class == ""] <- NA
maximizers$class2 = factor(maximizers$class2)
maximizers$class2 = relevel(maximizers$class2, ref="higher (1-5)")
xtabs(~maximizers$class2)

# recode gender
maximizers$gender2 = maximizers$gender
maximizers$gender2[maximizers$gender == ""] <- NA
maximizers$gender2 = factor(maximizers$gender2)
maximizers$gender2 = relevel(maximizers$gender2, ref="f")
xtabs(~maximizers$gender2)

# create new data frame
maximizers_rep = maximizers[,c(2, 6, 10, 11, 13:16)]
maximizers_rep = maximizers_rep[complete.cases(maximizers_rep), ]
maximizers_rep = maximizers_rep[is.finite(maximizers_rep$log_wordcount),]
maximizers_rep$role2_short = factor(maximizers_rep$role2, labels=c("Def", "Wit", "Jud", "Vic", "Law"))

# REPRODUCE INGER'S ANALYSIS
#--------------------------------------------------------------

# using the gamlss package
install.packages("gamlss")
library(gamlss)
m_rep_gamlss = gamlss(Maximizers ~ gender2 + class2 + role2 + year_cat + wordCount, 
					  data=na.omit(maximizers_rep), family=NBI)

# using glm.nb (in the MASS package)
library(MASS)

m_rep_mass = glm.nb(Maximizers ~ gender2 + class2 + role2 + year_cat + offset(log(wordCount)), 
					data=maximizers_rep)

# Effect display
library(effects)
plot(allEffects(m_rep_mass), ylim=log(c(.01, .05)), cols=4, rows=1, ylab="", main="")

car::Anova(m_rep_mass)

plot(allEffects(m_rep_mass, se=list(level=.95)), 
	 layout=c(4,1), grid=T, between=list(x=.5),
	 ylab="Rate of maximizers (percent)",
	 axes=list(y=list(lim=log(c(.01, .05)))))


#install.packages("brms")
library(brms)
#install.packages("tidyselect")
library(tidyselect)
#remove.packages("rstan")
#if (file.exists(".RData")) file.remove(".RData")
#install.packages("rstan", repos = "https://cloud.r-project.org/", dependencies = TRUE)
library(rstan)

# Xcode (from Appstore) needed for Mac for the code below

str(maximizers)
model_maximizers_1 = brm(Maximizers ~ gender + class + role2 + year_c + offset(log_wordcount),
              data=na.omit(maximizers), family=negbinomial(),
						 chains=3, cores=3, iter=3000, warmup=1000,
						 prior=c(prior(normal(-8, 2), class=Intercept),
						 		 prior(normal(0, 2), class=b)))

print(model_maximizers_1)

# auxiliary model to obtain estimates for the non-linear patterns that are evident when comparing the periods
# nb! for boosters, downtoners and intensifiers, increase "iter=" to 4000

model_maximizers_aux = brm(Maximizers ~ gender + class + role2 + (1|year_cat) + offset(log_wordcount),
						 data=na.omit(maximizers), family=negbinomial(),
						 chains=3, cores=3, iter=3000, warmup=1000,
						 prior=c(prior(normal(-8, 2), class=Intercept),
						 		 prior(normal(0, 2), class=b),
						 	 	 prior(normal(0, 2.5), class=sd)))

print(model_maximizers_aux)


# counterfactual plots
#-------------------------------------------------------------------------------

post = brms::posterior_samples(model_maximizers_1)
str(post)

# gender
male =   c(1, 1, .5,  1/5,1/5,1/5,1/5, 0)
female = c(1, 0, .5,  1/5,1/5,1/5,1/5, 0)
gender_est = rbind(exp(quantile(as.matrix(post[,1:8]) %*% male, c(.025, .25, .5, .75, .975))),
				   exp(quantile(as.matrix(post[,1:8]) %*% female, c(.025, .25, .5, .75, .975))))

# social class
high = c(1, .5, 0,  1/5,1/5,1/5,1/5, 0)
low =  c(1, .5, 1,  1/5,1/5,1/5,1/5, 0)
class_est = rbind(exp(quantile(as.matrix(post[,1:8]) %*% high, c(.025, .25, .5, .75, .975))),
				  exp(quantile(as.matrix(post[,1:8]) %*% low, c(.025, .25, .5, .75, .975))))

# role
def = c(1, .5, .5,  0,0,0,0, 0)
wit = c(1, .5, .5,  1,0,0,0, 0)
jud = c(1, .5, .5,  0,1,0,0, 0)
vic = c(1, .5, .5,  0,0,1,0, 0)
law = c(1, .5, .5,  0,0,0,1, 0)
role_est = rbind(exp(quantile(as.matrix(post[,1:8]) %*% def, c(.025, .25, .5, .75, .975))),
				 exp(quantile(as.matrix(post[,1:8]) %*% wit, c(.025, .25, .5, .75, .975))),
				 exp(quantile(as.matrix(post[,1:8]) %*% jud, c(.025, .25, .5, .75, .975))),
				 exp(quantile(as.matrix(post[,1:8]) %*% vic, c(.025, .25, .5, .75, .975))),
				 exp(quantile(as.matrix(post[,1:8]) %*% law, c(.025, .25, .5, .75, .975))))

# time
x_seq = c(seq(-1.6, 2.1, .1), 2.26)
year_est = matrix(NA, ncol=5, nrow=length(x_seq))
for(i in 1:length(x_seq)) year_est[i,] = exp(apply(as.matrix(post[,1:8]) %*% c(1, .5, .5, 1/5, 1/5, 1/5, 1/5, x_seq[i]), 2, quantile, c(.025, .25, .5, .75, .975)))

# periods (non-linear patterns from auxiliary model)
post_aux = brms::posterior_samples(model_maximizers_aux)
period_baseline = c(1, .5, .5,  1/5,1/5,1/5,1/5)
year_cat_est = matrix(NA, nrow=5, ncol=5)
for(i in 1:5) year_cat_est[i,] = exp(apply((as.matrix(post_aux[,1:7]) %*% period_baseline + post_aux[,9+i]), 2, quantile, c(.025, .25, .5, .75, .975)))

gender_est = gender_est*100000
class_est = class_est*100000
role_est = role_est*100000
year_est = year_est*100000
year_cat_est = year_cat_est*100000


# counterfactual plot
#source("./r_utils/L_axis.R")
#library(R.utils)
#source("~/Desktop/Merja+Ewa R/r_utils/L_axis.R")

y_limits = c(160, 330) # <--- nb! this applies to intensifiers
y_limits = c(17, 63) # <--- nb! this applies to maximizers for book
y_limits = c(120, 260) # <--- nb! this applies to boosters
y_limits = c(6, 45) # <--- nb! this applies to downtoners
y_limits = c(90, 190) # <--- nb! this applies to 'very'
y_limits = c(24, 77) # <--- nb! this applies to boosters without 'very'
y_limits = c(82, 152) # <--- nb! this applies to intensifiers without 'very'
y_limits = c(4, 34) # <--- nb! this applies to 'a little'
y_limits = c(2, 16) # <--- nb! this applies to downtoners without 'a little'
y_limits = c(3, 34) # <--- nb! this applies to diminishers
y_limits = c(1, 13) # <--- nb! this applies to minimizers
y_limits = c(-0.5, 4.5) # <--- nb! this applies to moderators
y_limits = c(-0.2, 2.4) # <--- nb! this applies to diminishers without 'a little'


#without “sty” and “axis_L” (cf. email Lukas); "r_utils is just a set of parameter settings for fine-tuning the plots. It isn’t really essential." ## sty and axis_L have been deleted from class and row below

# class
plot2 = xyplot(class_est[,3] ~ 1:2, ylim=y_limits, xlim=c(.5, 2.5), pch=16, col=1, cex=.6,
               scales=list(tck=.6, x=list(at=1:2, label=c("Higher    ", "    Lower"), cex=1),
                           y=list(at=c(20, 30, 40, 50, 60), cex=1)), par.settings=list(clip=list(panel="off")), 
               xlab="", ylab="",
               panel=function(x,y,...){
                 panel.xyplot(x,y,...)
                 panel.segments(x0=x, x1=x, y0=class_est[,2], y1=class_est[,4], lwd=1.5)
                 panel.segments(x0=x, x1=x, y0=class_est[,1], y1=class_est[,5], lwd=.5)
                 panel.text(x=.5, y=66, adj=0, label="Class", cex=1.3)
               })
# gender
plot1 = xyplot(gender_est[,3] ~ 2:1, ylim=y_limits, xlim=c(.5, 2.5), pch=16, col=1, cex=.6,
               scales=list(tck=.6, x=list(at=2:1, label=c("   Male", "Female   "), cex=1),
                           y=list(at=c(20, 30, 40, 50, 60), cex=1)), par.settings=list(clip=list(panel="off")),
               xlab="", ylab="",
               panel=function(x,y,...){
                 panel.xyplot(x,y,...)
                 panel.segments(x0=x, x1=x, y0=gender_est[,2], y1=gender_est[,4], lwd=1.5)
                 panel.segments(x0=x, x1=x, y0=gender_est[,1], y1=gender_est[,5], lwd=.5)
                 panel.text(x=.5, y=66, adj=0, label="Gender", cex=1.3)
               })

# role
plot3 = xyplot(role_est[,3] ~ 1:5, ylim=y_limits, xlim=c(.5, 5.5), pch=16, col=1, cex=.6,
			   scales=list(tck=.6, x=list(at=1:5, label=c("Def", "Wit", "Jud", "Vic", "Law"), cex=1),
			   			y=list(at=c(20, 30, 40, 50, 60), cex=1)), par.settings=list(clip=list(panel="off")), 
			   xlab="", ylab="",
			   panel=function(x,y,...){
			   	panel.xyplot(x,y,...)
			   	panel.segments(x0=x, x1=x, y0=role_est[,2], y1=role_est[,4], lwd=1.5)
			   	panel.segments(x0=x, x1=x, y0=role_est[,1], y1=role_est[,5], lwd=.5)
			   	panel.text(x=.5, y=66, adj=0, label="Role", cex=1.3)
			   })

# year
plot4 = xyplot(year_est[,3] ~ x_seq, ylim=y_limits, xlim=c(-1.6, 2.26), type="l", col=1,
			   scales=list(tck=.6, x=list(at=c(-1, 0, 1, 2), label=c("1750", "1800", "1850", "1900"), cex=1),
			   			y=list(at=c(20, 30, 40, 50, 60), cex=1)), par.settings=list(clip=list(panel="off")), 
			   xlab="", ylab=list(label="Estimated rate\n(per 100,000 words)", cex=1.3),
			   panel=function(x,y,...){
			   	panel.polygon(x=c(x, rev(x)), y=c(year_est[,1], rev(year_est[,5])), border=F, col="grey90")
			   	panel.polygon(x=c(x, rev(x)), y=c(year_est[,2], rev(year_est[,4])), border=F, col="grey75")
			   	panel.points(x=(c(1740, 1780, 1820, 1860, 1900)-1800)/50, y=year_cat_est[,3], pch=16, cex=.6, col="grey50")
			   	panel.segments(x0=(c(1740, 1780, 1820, 1860, 1900)-1800)/50,
			   				   x1=(c(1740, 1780, 1820, 1860, 1900)-1800)/50,
			   				   y0=year_cat_est[,2], y1=year_cat_est[,4], lwd=1.5, col="grey50")
			   	panel.segments(x0=(c(1740, 1780, 1820, 1860, 1900)-1800)/50,
			   				   x1=(c(1740, 1780, 1820, 1860, 1900)-1800)/50,
			   				   y0=year_cat_est[,1], y1=year_cat_est[,5], lwd=.5, col="grey50")
			   	panel.segments(x0=-1.6, x1=-1.6, y0=10, y1=20)
			   	panel.xyplot(x,y,...)
			   	panel.text(x=-1.6, y=66, adj=0, label="Time", cex=1.3)
			   })

# postscript("./figures/Maximizers_effect_display.eps", height=3, width=8.5, horizontal=F)
dev.off()
print(plot4, position=c(0,0,.4, 1), more=T)
print(plot3, position=c(.35,0,.7, 1), more=T)
print(plot2, position=c(.65,0,.85, 1), more=T)
print(plot1, position=c(.8,0,1, 1))


# interpretation based on ideal types
#-------------------------------------------------------------------------------
# Lukas S new code begins here - adapt it in accordance with results

coefs = fixef(model_maximizers_1)[,1]
coefs

# female / witness / lower class
exp(sum(coefs[c(1,4,3)]))*100000

# female / witness / higher class
exp(sum(coefs[c(1,4)]))*100000

# male / defendant / higher class
exp(sum(coefs[c(1,2)]))*100000

# female / defendant / higher class <-- for boosters without 'very,' since no female judges
exp(sum(coefs[c(1)]))*100000

# female / defendant / higher class <-- for intensifiers without 'very'
exp(sum(coefs[c(1)]))*100000

# female / judge / higher class <--- for downtoners, although no female judges exist
exp(sum(coefs[c(1,5)]))*100000

# male / lawyer / lower class
exp(sum(coefs[c(1,2,7,3)]))*100000

# male / victim / lower class
exp(sum(coefs[c(1,2,6,3)]))*100000

# male / witness / lower class
exp(sum(coefs[c(1,2,4,3)]))*100000

# male / witness / higher class <--- for diminishers without 'little'
exp(sum(coefs[c(1,2,4)]))*100000

# female / victim / lower class <--- for minimizers since no female lawyers or judges
exp(sum(coefs[c(1,6,3)]))*100000

# female / lawyer / lower class <--- for diminishers without 'little' although no female lawyers exist, and hardly any lawyers of lower class
exp(sum(coefs[c(1,7,3)]))*100000

# male / judge / higher class
exp(sum(coefs[c(1,2,5)]))*100000

# female / defendant / lower class <-- for diminishers without 'little'
exp(sum(coefs[c(1,3)]))*100000

# interpretation based on rate ratios
#-------------------------------------------------------------------------------

# Class: Higher / lower
round(exp(coefs[3]), 2)

# Gender: male / female
round(exp(-coefs[2]), 2)

# Role: defendant / witness
round(exp(-coefs[4]), 2)

# Role: defendant / lawyer
round(exp(-coefs[7]), 2)

# Role: witness / judge <-- for downtoners since def not the highest
round(exp(-coefs[4]+coefs[5]), 2)

# Role: judge / victim <-- for boosters without 'very,' since def not the highest
round(exp(-coefs[5]+coefs[6]), 2)

# Role: victim / judge <-- for diminishers, nb! coefficient 5 for judge in Nov 2020
round(exp(coefs[6]-coefs[5]), 2)

# Role: witness / lawyer <-- for 'very' 
round(exp(coefs[4]-coefs[7]), 2)

# Role: defendant / judge <-- for moderators 
round(exp(coefs[5]), 2)

# Year: 1900 / 1740 (0 = 1800, SD = 50)
round(exp(coefs[8]*3.2), 2)

# Year: 1913 / 1720 (0 = 1800, SD = 50) Lukas A.'s code for ratio 1913/1720
round(exp(coefs[8]*(1913-1720)/50),  2)

# Lukas S new code ends here
#------------------------------------------------------------------------------
